#!/bin/bash

echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
echo ""
echo "Running test $1 ..."
echo ""

if test -e results/$1.sh; then

	./results/$1.sh

else

	./scripts/runTest.sh $1

fi

valgrindCode=$?
resultCode=0

echo -e "  2. Test results:"

# if we have an error, there should not be an output file
# we have a result file with "Error" or "validation failed"
# for checking the programms stderr/stdout

# input should not produce some output
value=$(<results/$1)
if [[ "validation failed" == $value* ]] || [[ "Error: " ==  $value* ]]; then

	# but we have an output
	if   [ -e ausgabe_/$1 ]; then
		echo -e "    You have got a result file, but you should not have a result.\n";
		resultCode=1
	fi

	# we have no output, check for error message
	if   [ ! -e ausgabe_/$1 ]; then
		./boxpack tests/${1} tmp > errorStdOut
		./boxpack tests/${1} tmp 2> errorStdErr

		# check standard output
		if [[ "validation failed" == $errorStdOut* ]] && [[ "validation failed" == $value* ]]; then
			echo -e "    You have got the same error log as in the guidelines.\n";

		## check standard error
		elif [[ "Error: " == $errorStdErr* ]] && [[ "Error: " == $value* ]]; then
			echo -e "    You have got the same error log as in the guidelines.\n";

		else
			echo -e "    Your error differs from the guidelines\n";
		fi

		#rm tmp
		rm errorStdOut
		rm errorStdErr
	fi

else
	if  [ ! -e output/ausgabe_$1 ]; then
		echo -e "    You have got no result file, but you should have one.\n";
		resultCode=1
		echo -e "Test failed!\n"
		exit 1
	fi


	diff_res=$(diff output/ausgabe_$1 results/$1)

	if [ ${#diff_res} != "0" ]; then

		echo "Left side: your result | Right side: expected result ( | marks differing lines)"
		echo ""

		#echo diff_res
		diff -y -W 200 output/ausgabe_$1 results/$1
		exit 0
	else

		if [[ $1 == tLS* ]]
			then
			if diff -q -w <(sort output/ausgabe_$1) <(sort results/$1) > /dev/null; then
				echo -e "    Your result matches the target.\n"
			else
				echo -e "    Your result differs from the target.\n    Look at the diff-output above to check your result.\n"
				resultCode=1
			fi
		else
			if diff -q output/ausgabe_$1 results/$1 > /dev/null; then
				echo -e "    Your result matches the target.\n"
			else
				echo -e "    Your result differs from the target.\n    Look at the diff-output above to check your result.\n"
				resultCode=1
			fi
		fi
	fi
fi

if [[ $valgrindCode == 1 || $resultCode == 1 ]]; then
	echo -e "Test failed!\n"
	exit 1
else
	echo -e "Test succesful!\n"
	exit 0
fi
