#!/bin/bash

clear;

#set -e

echo "################################################################"
echo ""
echo "Deleting old Files"
echo ""
echo "################################################################"
echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
if [ -d $"tests" ]; then
	rm tests/*; 	echo "Files deleted in tests/"
else
	echo "No files in test/"
fi
if [ -d $"scripts" ]; then
	rm scripts/*; 	echo "Files deleted in scripts/"
else
	echo "No files in scripts/"
fi
if [ -d $"results" ]; then
	rm results/*; 	echo "Files deleted in results/"
else
	echo "No files in results/"
fi
if [ -d $"output" ]; then
	rm output/*; 	echo "Files deleted in output/"
else
	echo "No files in output/"
fi

echo "################################################################"
echo ""
echo "Check Prerequisites"
echo ""
echo "################################################################"
echo ""
echo "Check for GCC"
if ! type gcc >/dev/null ; then
	echo 'ERROR: gcc not found!'
	exit 1
fi
echo "  Done"
echo "Check for Valgrind"
if ! type valgrind >/dev/null ; then
	echo 'ERROR: valgrind not found!'
	exit 1
fi
echo "  Done"
echo "Check if source code exists (boxpack.c)"
if ! test -f boxpack.c ; then
	echo 'ERROR: Could not find source code (boxpack.c)'
	exit 1
fi
echo "  Done"
echo ""
echo "################################################################"
echo ""
echo "Compiling"
echo ""
echo "################################################################"

echo ""
echo "Warnings/Errors:"
if gcc -Wall -Wextra -std=c99 boxpack.c -g -o boxpack; then
	echo ""
	echo "None."
	echo ""
else
	echo ""
	echo "ERROR: Compiling failed"
	echo ""
	echo "################################################################"
	exit 1;
fi

echo "################################################################"
echo ""
echo "Downloading/Creating Test Cases"
echo ""
echo "################################################################"
echo ""

if ! test -e tests || ! test -e results || ! test -e data || ! test -e scripts; then

	echo 'Downloading test cases ...'
		cp -r download/* .
	echo '  Done'

	home=$(pwd)

	echo 'Altering test cases ...'
	for file in $(ls results/); do
		sed -i s@"\$HOME"@"$home"@g results/$file
	done

	for file in $(ls tests/); do
		sed -i s@"\$HOME"@"$home"@g tests/$file
	done
	echo '  Done'

else

	echo "Not necessary."

fi

echo ""
echo "################################################################"
echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
echo "################################################################"
echo ""
echo "Running tests..."
echo ""
echo "################################################################"
echo ""
echo "Check used libraries"
./scripts/cFileCheck.sh
echo ""

if ! test -e output; then
	mkdir output
fi



failedCounter=0
testCounter=0

for t in $(ls tests/); do

	if [[ "$t" != 'tPipe-grep-history' ]]; then
		./scripts/executeTest.sh $t
		failedCounter=$(($failedCounter+$?))
		testCounter=$(($testCounter+1))
	fi
done

echo -e "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n"

echo -e "Conclusion: $failedCounter out of $testCounter tests failed.\n"

echo -e "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n"

exit 1
